==> kernel-1.1.8.24-sources.jar: voxeet/com/sdk/core/VoxeetSdkTemplate.java <==
package voxeet.com.sdk.core;

import android.app.Application;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.util.Log;
import android.widget.Toast;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.neovisionaries.ws.client.WebSocketState;
import com.voxeet.android.media.MediaSDK;
import com.voxeet.kernel.BuildConfig;
import com.voxeet.kernel.R;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.EventBusException;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Callable;

import eu.codlab.simplepromise.Promise;
import eu.codlab.simplepromise.solve.ErrorPromise;
import eu.codlab.simplepromise.solve.PromiseExec;
import eu.codlab.simplepromise.solve.PromiseSolver;
import eu.codlab.simplepromise.solve.Solver;
import okhttp3.OkHttpClient;
import retrofit2.Retrofit;
import voxeet.com.sdk.core.abs.AbstractConferenceSdkService;
import voxeet.com.sdk.core.preferences.VoxeetPreferences;
import voxeet.com.sdk.core.services.AudioService;
import voxeet.com.sdk.core.services.ConferenceService;
import voxeet.com.sdk.core.services.MediaService;
import voxeet.com.sdk.core.services.ScreenShareService;
import voxeet.com.sdk.core.services.UserService;
import voxeet.com.sdk.events.error.SocketConnectErrorEvent;
import voxeet.com.sdk.events.success.SocketConnectEvent;
import voxeet.com.sdk.events.success.SocketStateChangeEvent;
import voxeet.com.sdk.factories.EventsFactory;
import voxeet.com.sdk.json.Event;
import voxeet.com.sdk.json.UserInfo;
import voxeet.com.sdk.models.TokenResponse;
import voxeet.com.sdk.models.UserTokenResponse;
import voxeet.com.sdk.utils.ConferenceListener;
import voxeet.com.sdk.utils.Twig;
import voxeet.com.sdk.utils.VoxeetServerUrl;

/**
 * Created by kevinleperf on 09/11/2017.
 */

public abstract class VoxeetSdkTemplate<T extends AbstractConferenceSdkService> implements ConferenceListener {
    static final long DEFAULT_TIMEOUT_MS = 60 * 1000;
    private UserInfo _user_info;
    private VoxeetWebSocket _voxeet_websocket;
    private VoxeetHttp _voxeet_http;

    protected Context _application_context;
    protected EventBus _event_bus;

    private Twig _twig;

    private VoxeetServerUrl _voxeet_server_url;

    private HashMap<Class<? extends AbstractVoxeetService>, AbstractVoxeetService> _services;
    private List<Solver<Boolean>> mWaitForLogInSocket;

    private VoxeetSdkTemplate() {

    }

    protected VoxeetSdkTemplate(@NonNull Application application_context,
                                @NonNull String appId,
                                @NonNull String password,
                                @NonNull UserInfo userInfo,
                                boolean debug) {
        this(application_context, appId, password, userInfo, null, null, debug);
    }

    protected VoxeetSdkTemplate(@NonNull Application application_context,
                                @NonNull String tokenAccess,
                                @Nullable Callable<String> tokenRefresh,
                                @NonNull UserInfo userInfo,
                                boolean debug) {
        this(application_context, null, null, userInfo, tokenAccess, tokenRefresh, debug);
    }

    private VoxeetSdkTemplate(@NonNull Application application_context,
                              @Nullable String appId,
                              @Nullable String password,
                              @NonNull UserInfo userInfo,
                              @Nullable String tokenAccess,
                              @Nullable Callable<String> tokenRefresh,
                              boolean debug) {
        this();

        mWaitForLogInSocket = new ArrayList<>();
        _application_context = application_context;
        _event_bus = EventBus.getDefault();

        //make sure the looper is the main looper for promise execution
        Promise.setHandler(new Handler(Looper.getMainLooper()));

        //call pre init - can instantiate specific variables
        onSdkPreinit();

        _user_info = userInfo;

        Log.d("VoxeetSDK", "VoxeetSdkTemplate: version := " + BuildConfig.VERSION_NAME);

        _twig = new Twig(2, "VoxeetSDK :: " + getTag(), false);

        _voxeet_server_url = new VoxeetServerUrl(this);

        _voxeet_websocket = new VoxeetWebSocket(application_context, this, _voxeet_server_url.getSocketUrl());


        VoxeetHttp.Builder builder = new VoxeetHttp.Builder()
                .setApplication(application_context)
                .setVoxeetSDK(this)
                .setServerUrl(_voxeet_server_url.getServerUrl())
                .setServerPort(_voxeet_server_url.getServerPort())
                .setAppId(appId)
                .setPassword(password)
                .setTokenAccess(tokenAccess)
                .setTokenRefresh(tokenRefresh)
                .setVoxeetServiceListener(new VoxeetHttp.VoxeetServiceListener() {

                    @Override
                    public void onIdentifySuccess(UserTokenResponse userToken, TokenResponse tokenResponse) {

                        _twig.d("onIdentifySuccess " + userToken + " " + tokenResponse + " " + userToken.getJwtUserToken());
                        //if (sdkConferenceService == null)
                        //    sdkConferenceService = new AbstractConferenceSdkService(applicationContext, conferenceListener);

                        _voxeet_websocket.connect(userToken.getUserToken(), userToken.getJwtUserToken());
                    }

                    @Override
                    public void onIdentifyError(String error) {
                        Log.e(getTag(), error);
                    }

                    @Override
                    public void onNetworkError(UnknownHostException error) {
                        _event_bus.post(new SocketStateChangeEvent(WebSocketState.CLOSED.name()));
                    }
                })
                .setDebug(debug);

        _voxeet_http = builder.build();


        _services = new HashMap<>();
        VoxeetPreferences.init(getApplicationContext());
        initServices();
    }

    public void resetVoxeetHttp() {
        _voxeet_http.FORCE_TOKEN_RESET();
    }


    //TODO check for logout ?
    public void logUser(@NonNull UserInfo userInfo) {
        _user_info = userInfo;

        openSdkSession();
    }

    public Promise<Boolean> logUserWithChain(@NonNull UserInfo userInfo) {
        _user_info = userInfo;
        return logCurrentlySelectedUserWithChain();
    }

    public Promise<Boolean> logCurrentlySelectedUserWithChain() {
        if(isSocketOpen()) {
            return new Promise<Boolean>(new PromiseSolver<Boolean>() {
                @Override
                public void onCall(@NonNull Solver<Boolean> solver) {
                    solver.resolve(true);
                }
            });
        }

        Promise<Boolean> promise = new Promise<>(new PromiseSolver<Boolean>() {
            @Override
            public void onCall(@NonNull Solver<Boolean> solver) {
                mWaitForLogInSocket.add(solver);

                openSdkSessionChain().then(new PromiseExec<Boolean, Object>() {
                    @Override
                    public void onCall(@Nullable Boolean result, @NonNull Solver<Object> solver) {

                    }
                }).error(new ErrorPromise() {
                    @Override
                    public void onError(@NonNull Throwable error) {
                        solver.reject(error);

                        for (Solver<Boolean> solve : mWaitForLogInSocket) {
                            solve.reject(error);
                        }

                        mWaitForLogInSocket.clear(); //TODO check for any other attempt ?
                    }
                });
            }
        });

        return promise;
    }

    /**
     * The method logout() must be defined for every SDKs
     */
    public abstract Promise<Boolean> logout();


    public abstract void onSdkPreinit();


    protected abstract void initServices();

    protected HashMap<Class<? extends AbstractVoxeetService>, AbstractVoxeetService> getServices() {
        return _services;
    }

    public T getConferenceService() {
        //explicitly cast, ConferenceSdkService is stored as AbstractConferenceSdkService
        return (T) getServiceForKlass(AbstractConferenceSdkService.class);
    }

    public ScreenShareService getScreenShareService() {
        return getServiceForKlass(ScreenShareService.class);
    }

    public AudioService getAudioService() {
        return getServiceForKlass(AudioService.class);
    }

    /**
     * Sends conference reminder.
     *
     * @param ids the ids
     */
    public void sendConferenceReminder(List<String> ids) {
        ConferenceService service = getServiceForKlass(ConferenceService.class);
        if (service != null) {
            service.sendReminder(ids);
        }
    }

    protected <T extends AbstractVoxeetService> T getServiceForKlass(Class<T> klass) {
        T service = null;
        Iterator<Class<? extends AbstractVoxeetService>> iterator = _services.keySet().iterator();
        while (iterator.hasNext()) {
            Class<? extends AbstractVoxeetService> next = iterator.next();
            AbstractVoxeetService item = _services.get(next);


            if (klass.isInstance(item)) {
                service = (T) item;
            }
        }

        if (service == null) {
            getTwig().e(klass.getSimpleName() + " not found in the list of services");
        }
        return service;
    }

    /**
     * Send a conference broadcast.
     *
     * @param message the message
     */
    public void sendConferenceMessage(String message) {
        AbstractConferenceSdkService service = getServiceForKlass(AbstractConferenceSdkService.class);
        if (service != null) {
            service.sendBroadcastMessage(message);
        }
    }

    /**
     * Gets upload token. Won't be able to upload files and such without it
     */

    public void getUploadToken() {
        UserService service = getServiceForKlass(UserService.class);
        if (service != null) {
            service.getUploadToken();
        }
    }


    public void send(Event command) {
        ObjectMapper om = new ObjectMapper();
        try {
            this.sendConferenceMessage(om.writeValueAsString(command));
        } catch (JsonProcessingException e) {
            Log.e(Event.TAG, "failed to send command", e);
        }
    }

    public Event decode(String message) {
        return EventsFactory.decode(message);
    }

    public Twig getTwig() {
        return _twig;
    }

    public Retrofit getRetrofit() {
        return _voxeet_http.getRetrofit();
    }

    public OkHttpClient getClient() {
        return _voxeet_http.getClient();
    }

    public Context getApplicationContext() {
        return _application_context;
    }

    public VoxeetServerUrl getVoxeetServerUrl() {
        return _voxeet_server_url;
    }

    /**
     * Reset services.
     */
    public void resetServices() {
        Iterator<Class<? extends AbstractVoxeetService>> iterator = _services.keySet().iterator();
        while (iterator.hasNext()) {
            Class<? extends AbstractVoxeetService> next = iterator.next();
            _services.get(next).resetService();
        }
    }

    public EventBus getEventBus() {
        return _event_bus;
    }

    protected VoxeetWebSocket getSocket() {
        return _voxeet_websocket;
    }

    protected VoxeetHttp getVoxeetHttp() {
        return _voxeet_http;
    }

    public void closeSocket() {
        _voxeet_websocket.close(true);
    }

    public boolean isSocketOpen() {
        return _voxeet_websocket.isOpen();
    }

    public void openSdkSession() {
        _voxeet_http.identify(_user_info);
    }

    private Promise<Boolean> openSdkSessionChain() {
        return _voxeet_http.identifyChain(_user_info);
    }

    @Override
    public Promise<Boolean> onCreationSuccess(String conferenceId) {
        AbstractConferenceSdkService service = getServiceForKlass(AbstractConferenceSdkService.class);

        if (service != null) return service.joinUsingConferenceId(conferenceId);
        else {
            return new Promise<>(new PromiseSolver<Boolean>() {
                @Override
                public void onCall(@NonNull Solver<Boolean> solver) {
                    solver.resolve(false);
                }
            });
        }
    }


    protected abstract String getTag();

    /**
     * Register Media. To be called before creating/joining any conference.
     *
     * @param context    the context
     */
    public boolean register(@NonNull Context context) {
        return register(context, null);
    }

    /**
     * Register Media. To be called before creating/joining any conference.
     *
     * @param context    the context
     * @param subscriber the subscriber
     */
    public boolean register(@NonNull Context context, @Nullable Object subscriber) {
        try {
            MediaSDK.setContext(context);

            if (null != subscriber && !getEventBus().isRegistered(subscriber))
                getEventBus().register(subscriber);
        } catch (EventBusException error) {
            //silent catch
        } catch (UnsatisfiedLinkError error) {
            Toast.makeText(getApplicationContext(), context.getString(R.string.register_conf_error), Toast.LENGTH_SHORT).show();
            return false;
        }

        return true;
    }

    /**
     * Unregister Media. To be called in the on destroy of your activty/fragment
     *
     * @param subscriber the subscriber
     */
    public void unregister(@NonNull Object subscriber) {

        if (getEventBus().isRegistered(subscriber))
            getEventBus().unregister(subscriber);
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(SocketConnectEvent event) {
        for (Solver<Boolean> solver : mWaitForLogInSocket) {
            solver.resolve(true);
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(SocketConnectErrorEvent event) {
        for (Solver<Boolean> solver : mWaitForLogInSocket) {
            solver.resolve(false);
        }
    }

    @NonNull
    public MediaService getMediaService() {
        return getServiceForKlass(MediaService.class);
    }
}

==> kernel-1.1.8.27-sources.jar: voxeet/com/sdk/core/VoxeetSdkTemplate.java <==
package voxeet.com.sdk.core;

import android.app.Application;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.util.Log;
import android.widget.Toast;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.neovisionaries.ws.client.WebSocketState;
import com.voxeet.android.media.MediaSDK;
import com.voxeet.kernel.BuildConfig;
import com.voxeet.kernel.R;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.EventBusException;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Callable;

import eu.codlab.simplepromise.Promise;
import eu.codlab.simplepromise.solve.ErrorPromise;
import eu.codlab.simplepromise.solve.PromiseExec;
import eu.codlab.simplepromise.solve.PromiseSolver;
import eu.codlab.simplepromise.solve.Solver;
import okhttp3.OkHttpClient;
import retrofit2.Retrofit;
import voxeet.com.sdk.core.abs.AbstractConferenceSdkService;
import voxeet.com.sdk.core.preferences.VoxeetPreferences;
import voxeet.com.sdk.core.services.AudioService;
import voxeet.com.sdk.core.services.ConferenceService;
import voxeet.com.sdk.core.services.MediaService;
import voxeet.com.sdk.core.services.ScreenShareService;
import voxeet.com.sdk.core.services.UserService;
import voxeet.com.sdk.events.error.SocketConnectErrorEvent;
import voxeet.com.sdk.events.success.SocketConnectEvent;
import voxeet.com.sdk.events.success.SocketStateChangeEvent;
import voxeet.com.sdk.factories.EventsFactory;
import voxeet.com.sdk.json.Event;
import voxeet.com.sdk.json.UserInfo;
import voxeet.com.sdk.models.TokenResponse;
import voxeet.com.sdk.models.UserTokenResponse;
import voxeet.com.sdk.utils.ConferenceListener;
import voxeet.com.sdk.utils.Twig;
import voxeet.com.sdk.utils.VoxeetServerUrl;

public abstract class VoxeetSdkTemplate<T extends AbstractConferenceSdkService> implements ConferenceListener {
    static final long DEFAULT_TIMEOUT_MS = 60 * 1000;
    private UserInfo _user_info;
    private VoxeetWebSocket _voxeet_websocket;
    private VoxeetHttp _voxeet_http;

    protected Context _application_context;
    protected EventBus _event_bus;

    private Twig _twig;

    private VoxeetServerUrl _voxeet_server_url;

    private HashMap<Class<? extends AbstractVoxeetService>, AbstractVoxeetService> _services;
    private List<Solver<Boolean>> mWaitForLogInSocket;

    private VoxeetSdkTemplate() {

    }

    protected VoxeetSdkTemplate(@NonNull Application application_context,
                                @NonNull String appId,
                                @NonNull String password,
                                @NonNull UserInfo userInfo,
                                boolean debug) {
        this(application_context, appId, password, userInfo, null, null, debug);
    }

    protected VoxeetSdkTemplate(@NonNull Application application_context,
                                @NonNull String tokenAccess,
                                @Nullable Callable<String> tokenRefresh,
                                @NonNull UserInfo userInfo,
                                boolean debug) {
        this(application_context, null, null, userInfo, tokenAccess, tokenRefresh, debug);
    }

    private VoxeetSdkTemplate(@NonNull Application application_context,
                              @Nullable String appId,
                              @Nullable String password,
                              @NonNull UserInfo userInfo,
                              @Nullable String tokenAccess,
                              @Nullable Callable<String> tokenRefresh,
                              boolean debug) {
        this();

        mWaitForLogInSocket = new ArrayList<>();
        _application_context = application_context;
        _event_bus = EventBus.getDefault();

        //make sure the looper is the main looper for promise execution
        Promise.setHandler(new Handler(Looper.getMainLooper()));

        //call pre init - can instantiate specific variables
        onSdkPreinit();

        _user_info = userInfo;

        Log.d("VoxeetSDK", "VoxeetSdkTemplate: version := " + BuildConfig.VERSION_NAME);

        _twig = new Twig(2, "VoxeetSDK :: " + getTag(), false);

        _voxeet_server_url = new VoxeetServerUrl(this);

        _voxeet_websocket = new VoxeetWebSocket(application_context, this, _voxeet_server_url.getSocketUrl());


        VoxeetHttp.Builder builder = new VoxeetHttp.Builder()
                .setApplication(application_context)
                .setVoxeetSDK(this)
                .setServerUrl(_voxeet_server_url.getServerUrl())
                .setServerPort(_voxeet_server_url.getServerPort())
                .setAppId(appId)
                .setPassword(password)
                .setTokenAccess(tokenAccess)
                .setTokenRefresh(tokenRefresh)
                .setVoxeetServiceListener(new VoxeetHttp.VoxeetServiceListener() {

                    @Override
                    public void onIdentifySuccess(UserTokenResponse userToken, TokenResponse tokenResponse) {

                        _twig.d("onIdentifySuccess " + userToken + " " + tokenResponse + " " + userToken.getJwtUserToken());
                        //if (sdkConferenceService == null)
                        //    sdkConferenceService = new AbstractConferenceSdkService(applicationContext, conferenceListener);

                        _voxeet_websocket.connect(userToken.getUserToken(), userToken.getJwtUserToken());
                    }

                    @Override
                    public void onIdentifyError(String error) {
                        Log.e(getTag(), error);
                    }

                    @Override
                    public void onNetworkError(UnknownHostException error) {
                        _event_bus.post(new SocketStateChangeEvent(WebSocketState.CLOSED.name()));
                    }
                })
                .setDebug(debug);

        _voxeet_http = builder.build();


        _services = new HashMap<>();
        VoxeetPreferences.init(getApplicationContext());
        initServices();
    }

    public void resetVoxeetHttp() {
        _voxeet_http.FORCE_TOKEN_RESET();
    }


    //TODO check for logout ?
    public void logUser(@NonNull UserInfo userInfo) {
        _user_info = userInfo;

        openSdkSession();
    }

    public Promise<Boolean> logUserWithChain(@NonNull UserInfo userInfo) {
        _user_info = userInfo;
        return logCurrentlySelectedUserWithChain();
    }

    public Promise<Boolean> logCurrentlySelectedUserWithChain() {
        if (isSocketOpen()) {
            return new Promise<Boolean>(new PromiseSolver<Boolean>() {
                @Override
                public void onCall(@NonNull Solver<Boolean> solver) {
                    solver.resolve(true);
                }
            });
        }

        Promise<Boolean> promise = new Promise<>(new PromiseSolver<Boolean>() {
            @Override
            public void onCall(@NonNull Solver<Boolean> solver) {
                mWaitForLogInSocket.add(solver);

                // the current list if not empty, we make sure that we are not calling it more than one 1 time
                if(mWaitForLogInSocket.size() == 1) {

                    openSdkSessionChain().then(new PromiseExec<Boolean, Object>() {
                        @Override
                        public void onCall(@Nullable Boolean result, @NonNull Solver<Object> solver) {
                            solver.resolve(result);

                            for (Solver<Boolean> solve : mWaitForLogInSocket) {
                                solve.resolve(result);
                            }

                            mWaitForLogInSocket.clear(); //TODO check for any other attempt ?
                        }
                    }).error(new ErrorPromise() {
                        @Override
                        public void onError(@NonNull Throwable error) {
                            solver.reject(error);

                            for (Solver<Boolean> solve : mWaitForLogInSocket) {
                                solve.reject(error);
                            }

                            mWaitForLogInSocket.clear(); //TODO check for any other attempt ?
                        }
                    });
                } else {
                    //nothing to do, it will be resolved later
                }
            }
        });

        return promise;
    }

    /**
     * The method logout() must be defined for every SDKs
     */
    public abstract Promise<Boolean> logout();


    public abstract void onSdkPreinit();


    protected abstract void initServices();

    protected HashMap<Class<? extends AbstractVoxeetService>, AbstractVoxeetService> getServices() {
        return _services;
    }

    public T getConferenceService() {
        //explicitly cast, ConferenceSdkService is stored as AbstractConferenceSdkService
        return (T) getServiceForKlass(AbstractConferenceSdkService.class);
    }

    public ScreenShareService getScreenShareService() {
        return getServiceForKlass(ScreenShareService.class);
    }

    public AudioService getAudioService() {
        return getServiceForKlass(AudioService.class);
    }

    /**
     * Sends conference reminder.
     *
     * @param ids the ids
     */
    public void sendConferenceReminder(List<String> ids) {
        ConferenceService service = getServiceForKlass(ConferenceService.class);
        if (service != null) {
            service.sendReminder(ids);
        }
    }

    protected <T extends AbstractVoxeetService> T getServiceForKlass(Class<T> klass) {
        T service = null;
        Iterator<Class<? extends AbstractVoxeetService>> iterator = _services.keySet().iterator();
        while (iterator.hasNext()) {
            Class<? extends AbstractVoxeetService> next = iterator.next();
            AbstractVoxeetService item = _services.get(next);


            if (klass.isInstance(item)) {
                service = (T) item;
            }
        }

        if (service == null) {
            getTwig().e(klass.getSimpleName() + " not found in the list of services");
        }
        return service;
    }

    /**
     * Send a conference broadcast.
     *
     * @param message the message
     */
    public void sendConferenceMessage(String message) {
        AbstractConferenceSdkService service = getServiceForKlass(AbstractConferenceSdkService.class);
        if (service != null) {
            service.sendBroadcastMessage(message);
        }
    }

    /**
     * Gets upload token. Won't be able to upload files and such without it
     */

    public void getUploadToken() {
        UserService service = getServiceForKlass(UserService.class);
        if (service != null) {
            service.getUploadToken();
        }
    }


    public void send(Event command) {
        ObjectMapper om = new ObjectMapper();
        try {
            this.sendConferenceMessage(om.writeValueAsString(command));
        } catch (JsonProcessingException e) {
            Log.e(Event.TAG, "failed to send command", e);
        }
    }

    public Event decode(String message) {
        return EventsFactory.decode(message);
    }

    public Twig getTwig() {
        return _twig;
    }

    public Retrofit getRetrofit() {
        return _voxeet_http.getRetrofit();
    }

    public OkHttpClient getClient() {
        return _voxeet_http.getClient();
    }

    public Context getApplicationContext() {
        return _application_context;
    }

    public VoxeetServerUrl getVoxeetServerUrl() {
        return _voxeet_server_url;
    }

    /**
     * Reset services.
     */
    public void resetServices() {
        Iterator<Class<? extends AbstractVoxeetService>> iterator = _services.keySet().iterator();
        while (iterator.hasNext()) {
            Class<? extends AbstractVoxeetService> next = iterator.next();
            _services.get(next).resetService();
        }
    }

    public EventBus getEventBus() {
        return _event_bus;
    }

    protected VoxeetWebSocket getSocket() {
        return _voxeet_websocket;
    }

    protected VoxeetHttp getVoxeetHttp() {
        return _voxeet_http;
    }

    public void closeSocket() {
        _voxeet_websocket.close(true);
    }

    public boolean isSocketOpen() {
        return _voxeet_websocket.isOpen();
    }

    public void openSdkSession() {
        _voxeet_http.identify(_user_info);
    }

    private Promise<Boolean> openSdkSessionChain() {
        return _voxeet_http.identifyChain(_user_info);
    }

    @Override
    public Promise<Boolean> onCreationSuccess(String conferenceId) {
        AbstractConferenceSdkService service = getServiceForKlass(AbstractConferenceSdkService.class);

        if (service != null) return service.joinUsingConferenceId(conferenceId);
        else {
            return new Promise<>(new PromiseSolver<Boolean>() {
                @Override
                public void onCall(@NonNull Solver<Boolean> solver) {
                    solver.resolve(false);
                }
            });
        }
    }


    protected abstract String getTag();

    /**
     * Register Media. To be called before creating/joining any conference.
     *
     * @param context the context
     */
    public boolean register(@NonNull Context context) {
        return register(context, null);
    }

    /**
     * Register Media. To be called before creating/joining any conference.
     *
     * @param context    the context
     * @param subscriber the subscriber
     */
    public boolean register(@NonNull Context context, @Nullable Object subscriber) {
        try {
            MediaSDK.setContext(context);

            if (null != subscriber && !getEventBus().isRegistered(subscriber))
                getEventBus().register(subscriber);
        } catch (EventBusException error) {
            //silent catch
        } catch (UnsatisfiedLinkError error) {
            Toast.makeText(getApplicationContext(), context.getString(R.string.register_conf_error), Toast.LENGTH_SHORT).show();
            return false;
        }

        return true;
    }

    /**
     * Unregister Media. To be called in the on destroy of your activty/fragment
     *
     * @param subscriber the subscriber
     */
    public void unregister(@NonNull Object subscriber) {

        if (getEventBus().isRegistered(subscriber))
            getEventBus().unregister(subscriber);
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(SocketConnectEvent event) {
        for (Solver<Boolean> solver : mWaitForLogInSocket) {
            solver.resolve(true);
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(SocketConnectErrorEvent event) {
        for (Solver<Boolean> solver : mWaitForLogInSocket) {
            solver.resolve(false);
        }
    }

    @NonNull
    public MediaService getMediaService() {
        return getServiceForKlass(MediaService.class);
    }
}
